#!/usr/bin/perl -w

# COPYRIGHT:
#  
# This software is Copyright (c) 2007 NETWAYS GmbH, Christian Doebler 
#                                <support@netways.de>
# 
# (Except where explicitly superseded by other copyright notices)
# 
# 
# LICENSE:
# 
# This work is made available to you under the terms of Version 2 of
# the GNU General Public License. A copy of that license should have
# been provided with this software, but in any event can be snarfed
# from http://www.fsf.org.
# 
# This work is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 or visit their web page on the internet at
# http://www.fsf.org.
# 
# 
# CONTRIBUTION SUBMISSION POLICY:
# 
# (The following paragraph is not intended to limit the rights granted
# to you to modify and distribute this software under the terms of
# the GNU General Public License and is only of importance to you if
# you choose to contribute your changes and enhancements to the
# community by submitting them to NETWAYS GmbH.)
# 
# By intentionally submitting any modifications, corrections or
# derivatives to this work, or any other work intended for use with
# this Software, to NETWAYS GmbH, you confirm that
# you are the copyright holder for those contributions and you grant
# NETWAYS GmbH a nonexclusive, worldwide, irrevocable,
# royalty-free, perpetual, license to use, copy, create derivative
# works based on those contributions, and sublicense and distribute
# those contributions and any derivatives thereof.
#
# Nagios and the Nagios logo are registered trademarks of Ethan Galstad.


=head1 NAME

noma_notifier.pl  -  NETWAYS Notification Manager - Notifier 

=head1 SYNOPSIS

noma_notifier.pl -H|--host=<host name>
             -S|--service=<service description>
             -c|--check-type=<check type (h|s)>
             -s|--status=<check status>
             -t|--datetime=<date time>
             -a|--host-alias=<host alias>
             -i|--host-address=<host ip>
             -o|--output=<host- or serviceoutput>
             -n|--notification-type=<notification type>
             [-h|--help] [-V|--version]
  
Sends notifications by using settings in database.

=head1 OPTIONS

=over 4

=item -H|--host=<name-or-ip>

Name of the affected host

=item -S|--service=<name-or-ip>

Name of the affected service

=item -c|--check-type=<check type (h|s)>

Type of check: h for host, s for service

=item -s|--status=<status>

Numeric status of check. (= check result)

=item -t|--datetime=<date time>

Date and time (string) when check took place.

=item -a|--host-alias=<host alias>

Host alias.

=item -i|--host-address=<host ip>

Host's IP address.

=item -o|--output=<host- or serviceoutput>

Host- or Serviceoutput

=item -n|--notification-type=<notification type>

Notification type.

=item -V|--version

Print version an exit.

=item -h|--help

Print help message and exit.

=cut


use strict;
use warnings;

use DBI;
use Getopt::Long;
use POSIX;
use noma_conf;

#use Data::Dumper;

my $versionStr = '0.0.3';


my %stati_service = (
	'OK'		=> 'on_ok',
	'WARNING'	=> 'on_warning',
	'CRITICAL'	=> 'on_critical',
	'UNKNOWN'	=> 'on_unknown'
);
my %stati_host = (
	'UP'			=> 'on_host_up',
	'UNREACHABLE'	=> 'on_host_unreachable',
	'DOWN'			=> 'on_host_down'
);
my %check_type_str = (
	'h' => 'Host',
	's' => 'Service'
);


my $host = '';
my $host_alias = '';
my $host_address = '';
my $service = '';
my $check_type = '';
my $status = '';
my $datetime = '';
my $output = '';
my $notification_type = '',
my $verbose = undef;
my $version = undef;
my $help = undef;

my $log_count = 0;
my @triesPerID;
my $additional_run = 0;

my $debug = undef;
my $do_not_send = undef;

my %dbConf = conf();


##############################################################################
# HANDLING OF COMMAND-LINE PARAMETERS
##############################################################################
Getopt::Long::Configure('bundling');
my $clps = GetOptions(

	"H|host=s"				=> \$host,
	"a|host-alias=s"		=> \$host_alias,
	"i|host-address=s"		=> \$host_address,
	"S|service=s"			=> \$service,
	"c|check-type=s"		=> \$check_type,
	"s|status=s"			=> \$status,
	"t|datetime=s"			=> \$datetime,
	"n|notification-type=s"	=> \$notification_type,
	"o|output=s"			=> \$output,
	"V|version"				=> \$version,
	"h|help"				=> \$help

);


# print version?
if (defined($version)) {
	print 'Version: ' . $versionStr . "\n";
	exit 0;
}


# check for required values
die ('No host specified!') if ($host eq '');
die ('No service specified!') if ($service eq '' && $check_type eq 's');
die ('No check type specified!') if ($check_type eq '');
die ('No status specified!') if ($status eq '');


# check for invalid values
if ($check_type eq 'h') {
	die ('Invalid check status!') unless (defined($stati_host{$status}));
} elsif ($check_type eq 's') {
	die ('Invalid check status!') unless (defined($stati_service{$status}));
} else {
	die ('Invalid check type!');	
}




##############################################################################
# GENERATE LIST OF CONTACTS TO NOTIFY
##############################################################################

# first check to prevent unwanted multi notifications
exit(0) if (nothingToDo());


# generate query and get list of possible users to notify
my $query = 'select id,hosts_include,hosts_exclude,services_include,services_exclude from notifications';
if ($check_type eq 'h') {
	$query .= ' where ' . $stati_host{$status} . '=\'1\'';
} else {
	$query .= ' where ' . $stati_service{$status} . '=\'1\'';
}
my %dbResult = queryDB($query);


# filter out unneeded users by using exclude lists
my @ids = generateNotificationList($check_type, $host, $service, %dbResult);
print "IDs collected (unfiltered): " . join('|', @ids) . "\n" if (defined($debug));


# no matches!?
my $idCount = @ids;
if ($idCount < 1) {
	print "No matches!\n" if (defined($debug));
	writeLog('no matches', '', '');
	exit;
}


# check escalations
my $notificationCounter = getNotificationCounter(@ids);
print "notification counter: $notificationCounter\n" if (defined($debug));


# get final contact data
my %contacts_c = getUsersAndMethods(@ids);
debugHash(%contacts_c) if (defined($debug));
my %contacts_cg = getUsersAndMethodsFromGroups(@ids);
debugHash(%contacts_cg) if (defined($debug));


# redo the contact data job if we are dealing with a recovery
if ($notificationCounter > 0) {

	# backup the notification counter to fake a brandnew notification
#	my $notificationCounter_tmp = $notificationCounter;
#	$notificationCounter = 0;

	$additional_run = 1;

	# get contacts and contactgroups... again
	my %contacts_c_tmp = getUsersAndMethods(@ids);
	debugHash(%contacts_c_tmp) if (defined($debug));
	my %contacts_cg_tmp = getUsersAndMethodsFromGroups(@ids);
	debugHash(%contacts_cg_tmp) if (defined($debug));

	# merge old and new data
	%contacts_c = mergeHashes(\%contacts_c, \%contacts_c_tmp);
	debugHash(%contacts_c) if (defined($debug));
	%contacts_cg = mergeHashes(\%contacts_cg, \%contacts_cg_tmp);
	debugHash(%contacts_cg) if (defined($debug));

#	# restore notification counter
#	$notificationCounter = $notificationCounter_tmp;

}


# merge contact hashes
my %contacts = mergeHashes(\%contacts_c, \%contacts_cg);
%contacts = removeHashEntryDuplicates(\%contacts);
debugHash(%contacts) if defined($debug);

# get current time
my $now = time;


# check contacts for holidays
%contacts = checkHolidays(%contacts);
debugHash(%contacts) if (defined($debug));


# check contacts for working hours
%contacts = checkWorkingHours(%contacts);
debugHash(%contacts) if (defined($debug));

# convert contact hash into array
my @contactsArr = hash2arr(%contacts);




##############################################################################
# SEND COMMANDS
##############################################################################

# loop through list of contacts
for my $contact (@contactsArr) {

	my $curUser = $contact->{username};
	my $curMethod = $contact->{method};
	my $curCmd = $contact->{command};
	my $curDest = $contact->{$contact->{contact_field}};
	my $curFrom = $contact->{from};


	# error if script is missing
	unless (-x $curCmd) {
		print "Missing script: $curUser!\n" if (defined($debug));
		writeLog('missing script', $curMethod, $curUser) ;
		next;		
	}

	# error if something is missing
	unless (defined($curCmd)) {
		print "Missing command for notification: $curUser!\n" if (defined($debug));
		writeLog('missing command for notification', $curMethod, $curUser) ;
		next;
	}
	unless (defined($curDest)) {
		print "Missing destination: $curUser!\n" if (defined($debug));
		writeLog('missing destination', $curMethod, $curUser);
		next;
	}


	# default 'from'
	unless (defined($curFrom)) {
		my $curFrom = '';
	}


	# calculate current time in user's timezone
	my $localNow = $now + ($contact->{time_diff} * 3600);

	# create call (COMMAND FROM DESTINATION CHECK-TYPE DATETIME STATUS NOTIFICATION-TYPE HOST-NAME HOST-ALIAS HOST-IP OUTPUT [SERVICE])
	my $curCall = sprintf("%s \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
		$curCmd,
		$curFrom,
		$curDest,
		$check_type,
		$datetime,
		$status,
		$notification_type,
		$host,
		$host_alias,
		$host_address,
		$output
	);
	$curCall .= " \"$service\"" if ($check_type eq 's');

	# send notification
	 if (defined($debug)) {
		print "Send: " . $contact->{username} . " | $curCmd | $curDest\n";
		print "Command: $curCall\n";
	 }
	`$curCall` unless (defined($do_not_send));

	# check whether sending was successful
	if ($?) {

		# sending was NOT successful

		# try to get next method (method escalation)
		if ($contact->{on_fail} eq '0') {

			writeLog('failed - no methods left', $curMethod, $curUser);
			print $contact->{username} . ": No methods left!\n" if defined($debug);

		} else {

			writeLog('failed - trying next method', $curMethod, $curUser);
			
			@contactsArr = getNextMethod($contact, \@contactsArr);

		}

	} else {

		# sending was successful -> write to log
		writeLog('successful', $curMethod, $curUser);

	}

}

# increment the notification counter, regardless of the last results
incrementNotificationCounter();




##############################################################################
# NOTIFICATION- AND CONTACT-FILTERING FUNCTIONS
##############################################################################
sub generateNotificationList {

	my ($check_type, $notificationHost, $notificationService, %dbResult) = @_;

	my $cnt = 0;
	my %hostList;
	my %serviceList;

	# BEGIN - generate include and exclude lists for hosts and services
	while (defined($dbResult{$cnt})) {

		# generate host-include list
		my @hosts = split(',', $dbResult{$cnt}->{hosts_include});

		# check each host against the host, passed with command-line parameters
		# and add it to the include list
		for my $host (@hosts) {

			$host =~ s/\*/.*/g;

			# only add host to list if it matches the passed one
			if ($notificationHost =~ m/^($host)$/) {
				print "Step1: HostIncl: $host\t" . $dbResult{$cnt}->{id} . "\n" if (defined($debug));
				$hostList{$dbResult{$cnt}->{id}} = 1;
			}

		}


		# remove hosts to be excluded
		@hosts = split(',', $dbResult{$cnt}->{hosts_exclude});

		# check each host against the host, passed with command-line parameters
		# and remove them from host list
		for my $host (@hosts) {

			$host =~ s/\*/.*/g;

			if ($notificationHost =~ m/^($host)$/) {
				print "Step1: HostExcl: $host\t" . $dbResult{$cnt}->{id} . "\n" if (defined($debug));
				undef($hostList{$dbResult{$cnt}->{id}}) if (defined($hostList{$dbResult{$cnt}->{id}}));
			}

		}


		if ($check_type eq 's') {

			# generate service-include list
			my @services = split(',', $dbResult{$cnt}->{services_include});

			# check each service against the service, passed with command-line parameters
			# and add it to the include list
			for my $service (@services) {

				$service =~ s/\*/.*/g;

				# only add service to list if it matches the passed one
				if ($notificationService =~ m/^($service)$/) {
					print "Step1: ServiceIncl: $service\t" . $dbResult{$cnt}->{id} . "\n" if (defined($debug));
					$serviceList{$dbResult{$cnt}->{id}} = 1;
				}

			}


			# remove services to be excluded
			@services = split(',', $dbResult{$cnt}->{services_exclude});

			# check each host against the service, passed with command-line parameters
			# and remove them from service list
			for my $service (@services) {

				$service =~ s/\*/.*/g;

				if ($notificationService =~ m/^($service)$/) {
					print "Step1: ServiceExcl: $service\t" . $dbResult{$cnt}->{id} . "\n" if (defined($debug));
					undef($serviceList{$dbResult{$cnt}->{id}}) if defined($serviceList{$dbResult{$cnt}->{id}});
				}

			}

		}

		# increase counter
		$cnt++;

	}
	# END   - generate include and exclude lists for hosts and services


	# BEGIN - collect all IDs to notify
	my %idList;
	my @ids;
	while (my ($hostIncl) = each(%hostList)) {
		print "Step2: HostIncl: $hostIncl\n" if (defined($debug));
		if ($check_type eq 's') {
			$idList{$hostIncl} = 1 if (defined($serviceList{$hostIncl}));
		} else {
			$idList{$hostIncl} = 1;
		}
	}
	if ($check_type eq 's') {
		while (my ($serviceIncl) = each(%serviceList)) {
			print "Step2: ServiceIncl: $serviceIncl\n" if (defined($debug));
			$idList{$serviceIncl} = 1 if (defined($hostList{$serviceIncl}));
		}
	}
	while (my ($id) = each(%idList)) {
		push(@ids, $id);
	}
	# END   - collect all IDs to notify

	return @ids;

}




sub getNotificationCounter {

	my @ids = @_;
	my $counter = 0;

	my $append = '';

	if ($status ne 'OK' && $status ne 'UP') {
		$append = ' and check_result=\'' . $status . '\'';
	}

	# determine status of check
	$query = 'select counter from notification_stati
				where host=\'' . $host . '\' and ' . 'service=\'' . $service . '\'
					and check_type=\'' . $check_type . '\'' . $append;
	my %dbResult = queryDB($query);

	$counter = $dbResult{0}->{counter} if (defined($dbResult{0}->{counter}));

	return $counter;
	
}




sub getUsersAndMethods {

	my @ids = @_;

	my %dbResult;
	my %dbResult_tmp;
	my %null;
	my %contacts;
	my @escalations;


	if ($notificationCounter > 0 && $additional_run != 1) {

		# possible escalations...
		for my $id (@ids) {

			# query for escalations
			if ($status ne 'OK' && $status ne 'UP') {

				# if there is no recovery, we are searching for exact matches in the notification count
				$query = 'select notify_after_tries from escalations_contacts
							where notification_id=\'' . $id . '\'
								and  notify_after_tries=\'' . $notificationCounter . '\'
								order by notify_after_tries asc limit 1';

			} else {

				# if we are dealing with a recovery, we have to notify everybody who has been notified until now
				$query = 'select notify_after_tries from escalations_contacts
							where notification_id=\'' . $id . '\'
								and  notify_after_tries < ' . $notificationCounter . '
								order by notify_after_tries asc limit 1';
				

			}
			%dbResult = queryDB($query);

			# store escalations if found
			if (defined($dbResult{0}->{notify_after_tries})) {

				# store escalation
				$triesPerID[$id] = $dbResult{0}->{notify_after_tries};

				# query for contacts
				$query = 'select distinct c.username, c.phone, c.mobile, c.email, tz.time_diff, m.id mid, m.method, m.command, m.contact_field, m.from, m.on_fail from escalations_contacts ec
							left join escalations_contacts_to_contacts ecc on ec.id=ecc.escalation_contacts_id
							left join contacts c on c.id=ecc.contacts_id
							left join escalations_contacts_to_methods ecm on ec.id=ecm.escalation_contacts_id
							left join notification_methods m on ecm.method_id=m.id
							left join timezones tz on c.timezone_id=tz.id
							left join notifications n on ec.notification_id=n.id
							where n.active=\'1\'
								and ec.notification_id=\'' . $id . '\'
								and ec.notify_after_tries=' . $dbResult{0}->{notify_after_tries};
				%dbResult_tmp = queryDB($query);

			} else {

				# store escalation
				$triesPerID[$id] = 0;

#				$query = 'select distinct c.username, c.phone, c.mobile, c.email, tz.time_diff, m.id mid, m.method, m.command, m.contact_field, m.from, m.on_fail from notifications n
#							left join notifications_to_methods nm on n.id=nm.notification_id
#							left join notification_methods m on m.id=nm.method_id
#							left join notifications_to_contacts nc on n.id=nc.notification_id
#							left join contacts c on c.id=nc.contact_id
#							left join timezones tz on c.timezone_id=tz.id
#							where n.active=\'1\' and n.id=\'' . $id . '\'';
#				%dbResult_tmp = queryDB($query);

			}

			# merge results
			%dbResult = mergeHashes(\%dbResult, \%dbResult_tmp);

		}

	} else {

		# no escalations -> standard query
		my $where = '';
		my $sep = '';
		for my $id (@ids) {
			$where .= $sep . 'n.id=\'' . $id . '\'';
			$sep = ' or ' if (!$sep); 
		}

		if ($where ne '') {

			if (defined($debug)) {		
				#$query = 'select distinct n.id,n.hosts_include from notifications n';
				$query = 'select distinct n.id, c.username, c.phone, c.mobile, c.email, tz.time_diff, m.id mid, m.method, m.command, m.contact_field, m.from, m.on_fail from notifications n';
			} else {
				$query = 'select distinct c.username, c.phone, c.mobile, c.email, tz.time_diff, m.id mid, m.method, m.command, m.contact_field, m.from, m.on_fail from notifications n';
			}

			my $where_after_tries = '';
			if ($status ne 'OK' && $status ne 'UP') {
				$where_after_tries = 'and n.notify_after_tries=\'' . $notificationCounter . '\'';
			}

			$query .= ' left join notifications_to_methods nm on n.id=nm.notification_id
							left join notification_methods m on m.id=nm.method_id
							left join notifications_to_contacts nc on n.id=nc.notification_id
							left join contacts c on c.id=nc.contact_id
							left join timezones tz on c.timezone_id=tz.id
							where n.active=\'1\' ' . $where_after_tries . ' and (' . $where . ')';

			%dbResult = queryDB($query);

		}

	}

	%dbResult = %null unless (defined($dbResult{0}->{username}));

	return %dbResult;

}




sub getUsersAndMethodsFromGroups {

	my @ids = @_;

	my %dbResult;
	my %dbResult_tmp;
	my %null;

	if ($notificationCounter > 0) {

		# possible escalations...
		for my $id (@ids) {

			if ($triesPerID[$id] != 0) {

				$query = 'select distinct c.username, c.phone, c.mobile, c.email, tz.time_diff, m.id mid, m.method, m.command, m.contact_field, m.from, m.on_fail from escalations_contacts ec
							left join escalations_contacts_to_contactgroups eccg on ec.id=eccg.escalation_contacts_id
							left join contactgroups_to_contacts cgc on eccg.contactgroup_id=cgc.contactgroup_id
							left join contacts c on cgc.contact_id=c.id
							left join escalations_contacts_to_methods ecm on ec.id=ecm.escalation_contacts_id
							left join notification_methods m on m.id=ecm.method_id
							left join timezones tz on c.timezone_id=tz.id
							left join notifications n on ec.notification_id=n.id
							where n.active=\'1\'
								and ec.notification_id=\'' . $id . '\'
								and ec.notify_after_tries=\'' . $triesPerID[$id] . '\'';
				%dbResult_tmp = queryDB($query);

			} else {

#				$query = 'select distinct c.username, c.phone, c.mobile, c.email, tz.time_diff, m.id mid, m.method, m.command, m.contact_field, m.from, m.on_fail from notifications n
#							left join notifications_to_methods nm on n.id=nm.notification_id
#							left join notification_methods m on m.id=nm.method_id
#							left join notifications_to_contactgroups ncg on n.id=ncg.notification_id
#							left join contactgroups cg on ncg.contactgroup_id=cg.id
#							left join contactgroups_to_contacts cgc on cgc.contactgroup_id=cg.id
#							left join contacts c on c.id=cgc.contact_id
#							left join timezones tz on c.timezone_id=tz.id
#							where n.active=\'1\' and n.id=\'' . $id . '\'';
#				%dbResult_tmp = queryDB($query);

			}

			# merge results
			%dbResult = mergeHashes(\%dbResult, \%dbResult_tmp);

		}

	} else {

		# no escalations -> standard query
		my $where = '';
		my $sep = '';
		for my $id (@ids) {
			$where .= $sep . 'n.id=\'' . $id . '\'';
			$sep = ' or ' if (!$sep); 
		}

		# query db for contacts
		if ($where ne '') {

			my $where_after_tries = '';
			if ($status ne 'OK' && $status ne 'UP') {
				$where_after_tries = 'and n.notify_after_tries=\'' . $notificationCounter . '\'';
			}

			$query = 'select distinct c.username, c.phone, c.mobile, c.email, tz.time_diff, m.id mid, m.method, m.command, m.contact_field, m.from, m.on_fail from notifications n
						left join notifications_to_methods nm on n.id=nm.notification_id
						left join notification_methods m on m.id=nm.method_id
						left join notifications_to_contactgroups ncg on n.id=ncg.notification_id
						left join contactgroups cg on ncg.contactgroup_id=cg.id
						left join contactgroups_to_contacts cgc on cgc.contactgroup_id=cg.id
						left join contacts c on c.id=cgc.contact_id
						left join timezones tz on c.timezone_id=tz.id
						where n.active=\'1\' ' . $where_after_tries . ' and (' . $where . ')';

			%dbResult = queryDB($query);

		}

	}

	%dbResult = %null unless (defined($dbResult{0}->{username}));

	return %dbResult;

}



sub checkHolidays {

	my %contacts = @_;

	# set up variables
	my $on_holiday;
	my %newContacts;

	# loop through contacts
	while (my ($contact) = each(%contacts)) {

		# init on-holiday flag
		$on_holiday = 0;

		# get holiday entries for current contact
		$query = 'select h.start,h.end from holidays h
					left join contacts c on c.id=h.contact_id
					where c.username=\'' . $contacts{$contact}->{username} . '\'';

		my %dbResult = queryDB($query);

		# check person's holiday data
		if (keys(%dbResult) > 0) {

			# loop through holiday entries
			while ((my ($dbCurrent) = each(%dbResult))) {

				# calculate current time in user's timezone
				my $localNow = $now + ($contacts{$contact}->{time_diff} * 3600);

				# convert start and end of holidays
				my $start = getUnixTime($dbResult{$dbCurrent}->{start});
				my $end = getUnixTime($dbResult{$dbCurrent}->{end});

				if (defined($debug)) {
					print "username: " . $contacts{$contact}->{username} . "\n";
					print "start:    $start\n";
					print "now:      $localNow\n";
					print "end:      $end\n";
				}

				# drop contact and break loop if on holidays
				if ($start < $localNow && $end > $localNow) {
					print "holiday drop: " . $contacts{$contact}->{username} . "\n" if (defined($debug));
					$on_holiday = 1;
					last;
				}

			}

		}

		# add contact to new conact list if not on holidays
		if (!$on_holiday) {
			$newContacts{$contact} = $contacts{$contact};
		}

	}

	return %newContacts;

}




sub checkWorkingHours {

	my %contacts = @_;

	# set up variables
	my $away;
	my %newContacts;

	# get current minute, hour and day of week
	my ($min, $hour, $day) = (localtime)[1,2,6];

	# set number indicating day of week
	my @weekArr = (0, 0, 0, 0, 0, 0, 0);
	$weekArr[$day] = 1;
	my $weekStr = join('', @weekArr);
	print "Today: $weekStr\n" if (defined($debug));
	my $week = oct("0b$weekStr");

	# calculate seconds by using current hour and minute
	my $nowHM = ($min * 60 + $hour * 3600);
	print "Now: $nowHM\n" if (defined($debug));


	# loop through contacts
	while (my ($contact) = each(%contacts)) {

		# init away flag
		$away = 0;

		# get holiday entries for current contact
		$query = 'select t.days,t.starttime,t.endtime from time_periods t
					left join contacts c on c.time_period_id=t.id
					where c.username=\'' . $contacts{$contact}->{username} . '\'';

		my %dbResult = queryDB($query);

		# check person's working-time data
		if (keys(%dbResult) > 0) {

			# calculate current time in user's timezone
			my $localNow = $nowHM + ($contacts{$contact}->{time_diff} * 3600);


			# check days: drop contact and break loop if away
			print $contacts{$contact}->{username} . ': ' . sprintf("%07b", $dbResult{0}->{days}) . "\n" if (defined($debug));
			if (($dbResult{0}->{days} | $week) != $dbResult{0}->{days}) {
				print "away drop (day): " . $contacts{$contact}->{username} . "\n" if (defined($debug));
				$away = 1;
				next;
			}


			# check working hours
			my ($shour, $smin, $ssec) = split(':', $dbResult{0}->{starttime});
			my ($ehour, $emin, $esec) = split(':', $dbResult{0}->{endtime});
			my $start = ($smin * 60 + $shour * 3600);
			my $end = ($emin * 60 + $ehour * 3600);
			print $contacts{$contact}->{username} . ": $start - $end\n" if (defined($debug));

			# drop contact and break loop if on holidays
			if ($start > $localNow || $end < $localNow) {
				print "away drop (time): " . $contacts{$contact}->{username} . "\n" if (defined($debug));
				$away = 1;
				next;
			}

		}

		# add contact to new conact list if not on holidays
		if (!$away) {
			$newContacts{$contact} = $contacts{$contact};
		}

	}
	
	return %newContacts;

}




sub getNextMethod {

	# get params
	my ($contactCur, $contacts) = @_;

	# init
	my $query = undef;
	my %dbResult;
	my $checkContact = undef;
	my $key = undef;
	my $value = undef;
	my $numKeys = 0;
	my @keepValues = ('mobile', 'phone', 'username', 'email', 'time_diff');

	# check whether next method is already scheduled and
	# return immediately if found
	for my $checkContact (@{$contacts}) {

		if (
			$checkContact->{username} eq $contactCur->{username} &&
			$checkContact->{mid} eq $contactCur->{on_fail}
			) {
				print "Method already scheduled!\n" if defined($debug);
				return @{$contacts};
		}
	}


	# get next escalation method
	$query = sprintf(
		'select m.id mid, m.method, m.command, m.contact_field, m.from, m.on_fail from notification_methods m where id=\'%s\'',
		$contactCur->{on_fail}
	);
	%dbResult = queryDB($query);


	# create temporary storage hash
	my %tmpHash;

	# copy old values
	foreach $key (@keepValues) {
		$tmpHash{$key} = $contactCur->{$key};
	}

	# set new values
	$tmpHash{mid} = $dbResult{0}->{mid};
	$tmpHash{on_fail} = $dbResult{0}->{on_fail};
	$tmpHash{from} = $dbResult{0}->{from};
	$tmpHash{method} = $dbResult{0}->{method};
	$tmpHash{contact_field} = $dbResult{0}->{contact_field};
	$tmpHash{command} = $dbResult{0}->{command};


	# store temporary hash in array
	push (@{$contacts}, {%tmpHash});

	# return new list of contacts
	return @{$contacts};

}




sub nothingToDo {

	# query for entries in the status history
	my $query = 'select count(*) cnt from notification_stati
				where host=\'' . $host . '\'
					and service=\'' . $service . '\'
					and check_type=\'' . $check_type . '\'';
	my %dbResult = queryDB($query);

	# there is nothing to to if the query resulted in zero and the check result was good
	return 1 if ($dbResult{0}->{cnt} eq '0' && ($status eq 'OK' || $status eq 'UP'));

	# 'something' to to
	return 0;

}




##############################################################################
# DATABASE FUNCTIONS
##############################################################################
sub queryDB {

	my $queryStr = shift;

	my $dbh = DBI->connect('DBI:mysql:host=' . $dbConf{host} . ';database=' . $dbConf{database}, $dbConf{user}, $dbConf{password});

	print "QUERY: " . $queryStr . "\n" if defined($debug);
	my $query = $dbh->prepare($queryStr);
	$query->execute;

	my %dbResult;
	my $cnt = 0;

	if ($dbh->rows && $queryStr =~ m/^select/) {
		while (my $row = $query->fetchrow_hashref) {
			$dbResult{$cnt++} = \%{$row};
		}
	}

	return %dbResult;

}




sub prepareDBValue {
	return str_replace("'", "\'", shift);
}




##############################################################################
# MISC FUNCTIONS
##############################################################################
sub debugArray {
	my @array = @_;
	for my $data (@array) {
		print "  $data\n"
	}
}




sub debugHash {
	my %hash = @_;
	while (my ($key, $value) = each %hash) {
		print "$key\n";
		while (my ($innerKey, $innerValue) = each %{$hash{$key}}) {
			print "     $innerKey - $innerValue\n";
		}
	}
}




sub removeHashEntryDuplicates {

	my ($inHash) = @_;

	my %outHash;
	my $found;
	my $countNew = 0;

	while (my ($entry) = each(%{$inHash})) {

		$found = 0;

		while (my ($entryOut) = each(%outHash)) {

			$found++ if (
				$inHash->{$entry}->{username} eq $outHash{$entryOut}->{username} &&
				$inHash->{$entry}->{method} eq $outHash{$entryOut}->{method}
			);

			last if ($found != 0);

		}

		if ($found == 0) {

			# store entry if it has not been found, yet
			$outHash{$countNew} = $inHash->{$entry};

			$countNew++;

		}

	}

	return %outHash;

}




sub mergeHashes {

	my ($hash1, $hash2) = @_;

	my %mergedHash;
	my $cnt = 0;

	# add hash1 to mergedHash
	while (my ($data) = each(%{$hash1})) {
		$mergedHash{$cnt++} = $hash1->{$data};
	}

	# add hash2 to mergedHash
	while (my ($data) = each(%{$hash2})) {
		$mergedHash{$cnt++} = $hash2->{$data};
	}

	return %mergedHash;

}




sub hash2arr {

	my (%hash) = @_;
	my @array;

	while(my ($data) = each(%hash)) {
		push(@array, $hash{$data});
	}

	return @array;

}




sub getUnixTime {

	my ($datetime) = @_;

	my ($date, $time) = split(' ', $datetime);
	my ($year, $mon, $day) = split('-', $date);
	my ($hour, $min, $sec) = split(':', $time);

	return mktime($sec, $min, $hour, $day, $mon - 1, $year - 1900);

}




sub incrementNotificationCounter {

	my $value;

	if ($status eq 'OK' || $status eq 'UP') {

		$query = 'delete from notification_stati
					where host=\'' . $host . '\'
					and service=\'' . $service . '\'
					and check_type=\'' . $check_type . '\'';
	
	} else {

		if ($notificationCounter == 0) {
			$query = 'insert into notification_stati (host,service,check_type,check_result,counter)
						values (' . "'$host','$service','$check_type','$status','1')";
		} else {
			$query = 'update notification_stati set counter=counter+1
						where host=\'' . $host . '\' and ' . 'service=\'' . $service . '\'
							and check_type=\'' . $check_type . '\' and check_result=\'' . $status . '\'';
		}

	}

	queryDB($query);

}




sub writeLog {

	my ($result, $method, $user) =  @_;

	# create timestamp
	my ($sec, $min, $hour, $day, $mon, $year) = (localtime)[0,1,2,3,4,5];
	$year += 1900;
	$mon++;
	my $timestamp = sprintf(
		"%d-%02d-%02d %02d:%02d:%02d",
		$year, $mon, $day, $hour, $min, $sec
	);

	my $query = sprintf(
		'insert into notification_logs (timestamp,counter,check_type,check_result,host,service,method,user,result)
			values (\'%s\',\'%s\',\'%s\',\'%s\',\'%s\',\'%s\',\'%s\',\'%s\',\'%s\')',
		$timestamp,
		$log_count++,
		$check_type_str{$check_type},
		$status,
		$host,
		$service,
		$method,
		$user,
		$result
	);

	queryDB($query);

}


# EOF
